"""
Builder package for SatVu API SDK.

This package generates Python SDK code from OpenAPI specifications.
"""

# Import patches to apply monkey-patches to openapi-python-client
import builder.patches  # noqa: F401
